package com.nisum.offertransactionservice.handler;



import com.nisum.offertransactionservice.genericexception.GlobalApiGenericException;
import com.safeway.epe.exception.ExceptionResponseHandlerImpl;
import com.safeway.epe.model.Response;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.ServletWebRequest;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

@Order(Ordered.HIGHEST_PRECEDENCE)
@ControllerAdvice
public class GlobalExceptionHandler extends ResponseEntityExceptionHandler {


    @Autowired
    ExceptionResponseHandlerImpl exceptionResponseHandler;

    @ExceptionHandler(GlobalApiGenericException.class)
    protected ResponseEntity<Response> handleEntityNotFound(GlobalApiGenericException ex, WebRequest webRequest) {
        Response errorResponse;
        if(ex.getResponse()!=null){
            errorResponse= ex.getResponse();
        }
        else if(ex.isHttpError()){
            errorResponse = exceptionResponseHandler.
                    createHttpErrorResponse(ex.getStatusCode(), "Http Error In Application",
                            ex, getUri((ServletWebRequest) webRequest), getHttpStatus(ex.getStatusCode()));
        }else{
            errorResponse = exceptionResponseHandler.
                    createApiErrorResponse(ex.getStatusCode(), "Search Result Not Found On Database",
                            ex, getUri((ServletWebRequest) webRequest), getHttpStatus(ex.getStatusCode()));
        }
        return new ResponseEntity<>(errorResponse,getHttpStatus(ex.getStatusCode()));
    }

    @Override
    protected ResponseEntity<Object> handleMethodArgumentNotValid(MethodArgumentNotValidException ex,
                                                                  HttpHeaders headers, HttpStatus status,
                                                                  WebRequest request) {
        final Response apiErrorResponse = exceptionResponseHandler.
                createApiErrorResponse("400", "Search Result Not Found On Database",
                        ex, getUri((ServletWebRequest) request), getHttpStatus("400"));
        return handleExceptionInternal(ex, apiErrorResponse, new HttpHeaders(), HttpStatus.BAD_REQUEST, request);
    }

    public HttpStatus getHttpStatus(String statusCode){
        HttpStatus status = HttpStatus.INTERNAL_SERVER_ERROR;
        if(statusCode.equalsIgnoreCase("400")){
            status = HttpStatus.NOT_FOUND;
        }
        return status;
    }

    private String getUri(ServletWebRequest webRequest) {
        return webRequest.getRequest().getRequestURL().toString();
    }


}
