package com.nisum.offertransactionservice.service;


import com.nisum.offertransactionservice.client.ClientService;
import com.nisum.offertransactionservice.converter.OfferConvertion;
import com.nisum.offertransactionservice.dao.OfferLookupRepo;
import com.nisum.offertransactionservice.dao.OfferMetaDataRepo;
import com.nisum.offertransactionservice.dto.*;
import com.nisum.offertransactionservice.genericexception.GlobalApiGenericException;
import com.nisum.offertransactionservice.model.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
@PropertySource("classpath:application.properties")
@Slf4j
public class OfferCallingPEService {


    @Autowired
    private OfferLookupRepo offerLookupRepo;

    @Autowired
    private ClientService clientService;

    @Autowired
    private OfferMetaDataRepo offerMetaDataRepo;

    public OfferTransactionResponse getDiscountedItemList(OfferTransactionRequest offerTransactionRequest) {
        log.info("Inside getDiscountedItemList Method");
        List<OfferLookup> eligibleOffer = new ArrayList();
        offerLookupRepo.findByHhId(offerTransactionRequest.getHhId()).forEach(eligibleOffer::add);
        List<OfferLookupDTO> offerLookupDTOList = getOfferMetaDto(eligibleOffer);
        if (offerLookupDTOList.isEmpty()) {
            log.error("Offer lookup Object is empty");
            throw new GlobalApiGenericException("Unable to get the Offer Response ", new Exception("exception e"), "400", false);
        }
        log.debug("Offer lookup Object {}", offerLookupRepo);
        OfferTransactionResponse offerTransactionResponse;
        PERequest peRequest = OfferConvertion.INSTANCE.map(offerTransactionRequest);
        peRequest.setEligibleOffers(offerLookupDTOList);
        log.info("Promotional Engine WebClient call Start");
        PEResponse peResponseFlux = clientService.getPeResponseFlux(peRequest);
        log.debug("Promotional Engine WebClient call End");
        offerTransactionResponse = OfferConvertion.INSTANCE.map(peResponseFlux);
        offerTransactionResponse.setTransactionId(offerTransactionRequest.getTransactionId());
        log.debug("Offer Transaction Response {}", offerTransactionResponse);
        return offerTransactionResponse;
    }

    public List<OfferLookupDTO> getOfferMetaDto(List<OfferLookup> eligibleOffer){

        return eligibleOffer.stream().
                map(offer -> {
                     Optional<OfferMeta>  optionalOfferMeta = offerMetaDataRepo.
                             findByOfferIdAndOfferStatusIgnoreCase(offer.getOfferId(), "ACTIVE");

                     if(optionalOfferMeta.isPresent()){
                         OfferMetaDTO offerMetaDTO = OfferConvertion.INSTANCE.map(optionalOfferMeta.get());
                         OfferLookupDTO offerLookupDTO = OfferConvertion.INSTANCE.map(offer);
                         offerLookupDTO.setOfferMetaDTO(offerMetaDTO);
                         return offerLookupDTO;
                     }
                     return null;
                })
                .filter(offerLookupDTO -> offerLookupDTO!=null)
                .collect(Collectors.toList());

    }




}
