package com.nisum.offertransactionservice.service;

import com.nisum.offertransactionservice.dao.EndOfSaleRepo;
import com.nisum.offertransactionservice.dto.EndOfSaleReq;
import com.nisum.offertransactionservice.dto.Item;
import com.nisum.offertransactionservice.dto.OfferTransactionResponse;
import com.nisum.offertransactionservice.model.*;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.*;
import org.mockito.junit.MockitoJUnitRunner;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@RunWith(MockitoJUnitRunner.class)
public class EndOfSaleTest {

    @Mock
    private EndOfSaleService endOfSaleServiceMock;


    @InjectMocks
    private EndOfSaleService endOfSaleService;

    @Mock
    private EndOfSaleRepo endOfSaleRepo;

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void putEndofSaleResInDbTest() throws IOException {
        EndOfSaleReq endOfSaleReq = getEndOfSaleReq();
        Mockito.when(endOfSaleRepo.save(ArgumentMatchers.any())).thenReturn(ArgumentMatchers.any());
        String uuid = endOfSaleService.putEndofSaleResInDb(endOfSaleReq);
        Assert.assertEquals(uuid,"123456788899999998");
    }

    @Test
    public void putEndofSaleResInDbNegTest() throws IOException {
        EndOfSaleReq endOfSaleReq = getEndOfSaleNegReq();
        Mockito.when(endOfSaleRepo.save(ArgumentMatchers.any())).thenReturn(ArgumentMatchers.any());
        String uuid = endOfSaleService.putEndofSaleResInDb(endOfSaleReq);
        Assert.assertNotNull(uuid);
    }

    private EndOfSaleReq getEndOfSaleReq() {
        EndOfSaleReq endOfSaleReq = new EndOfSaleReq();
        endOfSaleReq.setOfferTransactionResponse(retOfferTransRes());
        endOfSaleReq.setOffers(returnOffers());
        return endOfSaleReq;
    }
    private EndOfSaleReq getEndOfSaleNegReq() {
        EndOfSaleReq endOfSaleReq = new EndOfSaleReq();
        endOfSaleReq.setOfferTransactionResponse(retOfferTransNegRes());
        endOfSaleReq.setOffers(returnOffers());
        return endOfSaleReq;
    }

    private OfferTransactionResponse retOfferTransRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        offerTransactionResponse.setTransactionId("123456788899999998");
        return offerTransactionResponse;
    }

    private OfferTransactionResponse retOfferTransNegRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        //offerTransactionResponse.setTransactionId("123456788899999998");
        return offerTransactionResponse;
    }

    public List<OfferLookup> returnOffers() {
        OfferLookup offerLookup1 = new OfferLookup("1", "12", 23L, "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup("2", "13", 24L, "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups.stream().collect(Collectors.toList());
    }

}
