package com.nisum.offertransactionservice.service;


import com.nisum.offertransactionservice.client.ClientService;
import com.nisum.offertransactionservice.dao.OfferLookupRepo;
import com.nisum.offertransactionservice.dao.OfferMetaDataRepo;
import com.nisum.offertransactionservice.dto.*;
import com.nisum.offertransactionservice.genericexception.GlobalApiGenericException;
import com.nisum.offertransactionservice.model.*;
import org.assertj.core.util.Lists;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.*;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.DirtiesContext;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@RunWith(MockitoJUnitRunner.class)
@SpringBootTest
public class OfferServiceTest {
    @Mock
    private OfferLookupRepo offerLookupRepo;

    @Mock
    private OfferMetaDataRepo offerMetaDataRepo;


    @InjectMocks
    private OfferCallingPEService offerCallingPEService = new OfferCallingPEService();

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Mock
    ClientService clientService;

    @Test
    public void serviceTest() {
        OfferTransactionRequest offerTransactionRequest = getOfferTransactionRequest();
        Mockito.when(offerLookupRepo.findByHhId(ArgumentMatchers.any())).thenReturn(returnOffers());
        Mockito.when(clientService.getPeResponseFlux(ArgumentMatchers.any())).thenReturn(getPeResponse());
        Mockito.doReturn(getOfferMeta()).when(offerMetaDataRepo).findByOfferIdAndOfferStatusIgnoreCase(ArgumentMatchers.any(),ArgumentMatchers.any());
        OfferTransactionResponse offerTransactionResponse = offerCallingPEService.getDiscountedItemList(offerTransactionRequest);
        Assert.assertNotNull(offerTransactionResponse);
    }

    @Test(expected = GlobalApiGenericException.class)
    public void serviceNegTest() {
        OfferTransactionRequest offerTransactionRequest = getOfferTransactionRequest();
        Mockito.when(offerLookupRepo.findByHhId(ArgumentMatchers.any())).thenReturn(returnOffers());
        Mockito.doReturn(Optional.empty()).when(offerMetaDataRepo).findByOfferIdAndOfferStatusIgnoreCase(ArgumentMatchers.any(),ArgumentMatchers.any());
        offerCallingPEService.getDiscountedItemList(offerTransactionRequest);
    }


    private OfferTransactionResponse retOfferTransRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        return offerTransactionResponse;
    }

    private PEResponse getPeResponse() {
        PEResponse peResponse = new PEResponse();
        peResponse.setHhid(123L);
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        peResponse.setDiscountedItemList(itemList);
        return peResponse;
    }

    private PERequest getPEReq() {
        PERequest peRequest = new PERequest();
        peRequest.setHhId(123L);
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        peRequest.setItemList(itemList);
        return peRequest;
    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionRequest.setItemList(itemList);
        return offerTransactionRequest;
    }

    public List<OfferLookup> returnOffers() {
        OfferLookup offerLookup1 = new OfferLookup("1", "12", 20L, "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup("2", "13", 23L, "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups.stream().collect(Collectors.toList());
    }



    private Optional<OfferMeta> getOfferMeta(){
        OfferMeta offerMeta = new OfferMeta();
        offerMeta.setDiscount("discount");
        offerMeta.setEligibility("eligibility");
        offerMeta.setExpiryTime(LocalDateTime.of(2021,01,01,01,01));
        offerMeta.setStartTime(LocalDateTime.of(2020,01,01,01,01));
        offerMeta.setOfferDesc("30 percent");
        offerMeta.setOfferId(1l);
        offerMeta.setOfferStatus("Active");
        offerMeta.setRcptTxt("text");
        offerMeta.setOfferType("aaa");

        return Optional.of(offerMeta);
    }

    private List<OfferLookupDTO> getOfferLookupDto(){
        OfferLookupDTO offerLookupDTO = new OfferLookupDTO();
        offerLookupDTO.setId("12345");
        offerLookupDTO.setIdType("1234");
        offerLookupDTO.setOfferId(1L);
        offerLookupDTO.setPreCondition("And");
        offerLookupDTO.setStoreId("001");
        offerLookupDTO.setTerminal("001");

        OfferMetaDTO offerMetaDTO = new OfferMetaDTO();
        offerMetaDTO.setDiscount("discount");
        offerMetaDTO.setEligibility("eligibility");
        offerMetaDTO.setExpiryTime(LocalDateTime.of(2021,01,01,01,01));
        offerMetaDTO.setStartTime(LocalDateTime.of(2020,01,01,01,01));
        offerMetaDTO.setOfferDesc("30 percent");
        offerMetaDTO.setOfferId(1L);
        offerMetaDTO.setOfferStatus("Active");
        offerMetaDTO.setRcptTxt("text");
        offerMetaDTO.setOfferType("aaa");

        offerLookupDTO.setOfferMetaDTO(offerMetaDTO);


        OfferLookupDTO offerLookupDTO1 = new OfferLookupDTO();
        offerLookupDTO1.setId("12345");
        offerLookupDTO1.setIdType("1234");
        offerLookupDTO1.setOfferId(1L);
        offerLookupDTO1.setPreCondition("And");
        offerLookupDTO1.setStoreId("001");
        offerLookupDTO1.setTerminal("001");

        OfferMetaDTO offerMetaDTO1 = new OfferMetaDTO();
        offerMetaDTO1.setDiscount("discount");
        offerMetaDTO1.setEligibility("eligibility");
        offerMetaDTO1.setExpiryTime(LocalDateTime.of(2021,01,01,01,01));
        offerMetaDTO1.setStartTime(LocalDateTime.of(2020,01,01,01,01));
        offerMetaDTO1.setOfferDesc("20 percent");
        offerMetaDTO1.setOfferId(1L);
        offerMetaDTO1.setOfferStatus("Active");
        offerMetaDTO1.setRcptTxt("text");
        offerMetaDTO1.setOfferType("aaa");

        offerLookupDTO1.setOfferMetaDTO(offerMetaDTO);


        List<OfferLookupDTO> offerLookupDTOList = Arrays.asList(offerLookupDTO1, offerLookupDTO);
        return offerLookupDTOList;


    }




}
